#include <ctype.h>
#include <iostream.h>

#include <qbuttongroup.h> 
#include <qlabel.h>
#include <qlayout.h>
#include <qmsgbox.h> 
#include <qradiobutton.h>  
#include <qsize.h>
#include <qvalidator.h> 

#include <kapp.h>
#include <kbuttonbox.h>
#include <kseparator.h>

#include "dialog.h"


CGotoDialog::CGotoDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder ) 
{
  QVBoxLayout *topLayout = new QVBoxLayout( this, 5 ); 
  QHBoxLayout *inputRow;

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  QLabel *label = new QLabel( i18n("Goto Offset"), this );
  label->setFixedHeight( label->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( label );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mComboBox = new QComboBox( true, this );
  mComboBox->setMaxCount( 10 );
  mComboBox->setInsertionPolicy( QComboBox::AtTop );
  mComboBox->setFixedHeight( mComboBox->sizeHint().height() );
  mComboBox->setMinimumWidth( mComboBox->fontMetrics().width("M") * 20 );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mComboBox );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  QButtonGroup *group = new QButtonGroup( i18n("Options"), this );  

  QGridLayout *gbox = new QGridLayout( group, 3, 2, 10 );
  gbox->addRowSpacing( 0, 0 );
  mCheckBackward = new QCheckBox( i18n("&Backwards"), group );
  mCheckBackward->setMinimumSize( mCheckBackward->sizeHint() );
  gbox->addWidget( mCheckBackward, 1, 0 );
  mCheckFromCursor = new QCheckBox( i18n("&From Cursor"), group );
  gbox->addWidget( mCheckFromCursor, 2, 0 );
  mCheckFromCursor->setMinimumSize( mCheckFromCursor->sizeHint() );
  gbox->activate();
  
  inputRow->addSpacing( 5 );
  inputRow->addWidget( group );
  inputRow->addSpacing( 5 );


  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 5, 5 );
  buttonBox->addStretch();
  QPushButton *gotoButton = buttonBox->addButton( i18n("&Apply"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("&Close"), false );
  buttonBox->layout();
  gotoButton->setDefault( true );
  buttonBox->setMinimumSize( gotoButton->sizeHint() );
  connect( gotoButton, SIGNAL(clicked()),this,SLOT(startGoto()) );
  connect( closeButton, SIGNAL(clicked()),this,SLOT(reject()) );
  topLayout->addWidget( buttonBox );


  topLayout->activate();
  topLayout->freeze( minimumSize().width(), minimumSize().height() );

  defaultFocus();
}

CGotoDialog::~CGotoDialog( void )
{

}




void CGotoDialog::defaultFocus( void )
{
  mComboBox->setFocus();
}


//
// Format of input string:
// 0x<HexNumber>|<DecimalNumber>s<Bit>
// s = :,. or space
//
void CGotoDialog::startGoto( void )
{
  USearchData sd;

  bool success = stringToOffset( mComboBox->currentText(), sd.mGoto.offset );
  if( success == false )
  {
    return;
  }
  
  sd.mGoto.operation  = USearchData::opGoto;
  sd.mGoto.bit        = 7;
  sd.mGoto.backward   = mCheckBackward->isChecked();
  sd.mGoto.fromCursor = mCheckFromCursor->isChecked();
  emit execute( sd );

  #if 0
  const char *p = mComboBox->currentText();
  if( strlen( p ) == 0 )
  {
    return;
  }

  //
  // Skip any whitespaces in front of string
  //
  for( ; *p != 0 && isspace( *p ) ; p++ );
  
  uint offset, bit;
  int match;
  if( strncmp( p, "0x", 2 ) == 0 || strncmp( p, "0X", 2 ) == 0 )
  {
    match = sscanf( p+2, "%x", &offset );
  }
  else
  {
    match = sscanf( p, "%u", &offset );
  }

  if( match == 0 )
  {
    return;
  }

  bit = 7;

  p = strpbrk( p, ":,. " );
  if( p != 0 )
  {
    match = sscanf( p+1, "%u", &bit );
    if( match == 0 )
    {
      return;
    }
    if( bit > 7 ) { bit = 7; }
  }

  USearchData sd;
  sd.mGoto.operation  = USearchData::opGoto;
  sd.mGoto.offset     = offset;
  sd.mGoto.bit        = bit;
  sd.mGoto.backward   = mCheckBackward->isChecked();
  sd.mGoto.fromCursor = mCheckFromCursor->isChecked();
  emit execute( sd );
  #endif
}




CFindDialog::CFindDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder ) 
{
  QVBoxLayout *topLayout = new QVBoxLayout( this, 5 ); 
  QHBoxLayout *inputRow;
  QLabel *label;

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  label = new QLabel( i18n("Format"), this );
  label->setFixedHeight( label->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( label );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mSelector = new QComboBox( false, this );
  mSelector->setFixedHeight( mSelector->sizeHint().height() );
  mSelector->setMinimumWidth( mSelector->fontMetrics().width("M") * 20 );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mSelector );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  label = new QLabel( i18n("Find"), this );
  label->setFixedHeight( label->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( label );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mInput = new QLineEdit( this );
  mInput->setFixedHeight( mInput->sizeHint().height() );
  mInput->setMinimumWidth( mInput->fontMetrics().width("M") * 20 );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mInput );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  QButtonGroup *group = new QButtonGroup( i18n("Options"), this );  
  QGridLayout *gbox = new QGridLayout( group, 3, 2, 10 );
  gbox->addRowSpacing( 0, 0 );
  mCheckBackward = new QCheckBox( i18n("&Backwards"), group );
  mCheckBackward->setMinimumSize( mCheckBackward->sizeHint() );
  gbox->addWidget( mCheckBackward, 1, 0 );
  mCheckFromCursor = new QCheckBox( i18n("&From Cursor"), group );
  gbox->addWidget( mCheckFromCursor, 2, 0 );
  mCheckFromCursor->setMinimumSize( mCheckFromCursor->sizeHint() );
  mCheckInSelection = new QCheckBox( i18n("&In Selection"), group );
  mCheckInSelection->setMinimumSize( mCheckInSelection->sizeHint() );
  gbox->addWidget( mCheckInSelection, 1, 1 );
  gbox->activate();
  inputRow->addSpacing( 5 );
  inputRow->addWidget( group );
  inputRow->addSpacing( 5 );

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 5, 5 );
  buttonBox->addStretch();
  //QPushButton *againButton = buttonBox->addButton( i18n("&Again"), false );
  QPushButton *findButton = buttonBox->addButton( i18n("&Apply"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("&Close"), false );
  buttonBox->layout();
  findButton->setDefault( true );
  buttonBox->setMinimumSize( findButton->sizeHint() );
  //connect( againButton, SIGNAL(clicked()),this,SLOT(findAgain()) );
  connect( findButton, SIGNAL(clicked()),this,SLOT(startFind()) );
  connect( closeButton, SIGNAL(clicked()),this,SLOT(reject()) );
  topLayout->addWidget( buttonBox );


  topLayout->activate();
  topLayout->freeze( minimumSize().width(), minimumSize().height() );

  const char *items[] = 
  { 
    i18n( "Regular text" ),
    i18n( "Hexadecimal" ),
    i18n( "Octal" ),
    i18n( "Binary" ),
    0 
  };
  mSelector->insertStrList( items );
  mFindValidator = new CHexValidator( this, CHexValidator::regularText );
  mInput->setValidator( mFindValidator );

  connect( mSelector, SIGNAL(activated(int)), 
	   SLOT(selectorChanged(int)) );
  connect( mInput, SIGNAL(textChanged(const char*)), 
	   SLOT(inputChanged(const char*)) );
  defaultFocus();
}


CFindDialog::~CFindDialog( void )
{
  delete mFindValidator;
}


void CFindDialog::selectorChanged( int index )
{
  mFindValidator->setState( (CHexValidator::EState)index );
  mInput->setText( mFindString[ index ] );
}

void CFindDialog::inputChanged( const char *text )
{
  mFindString[ mSelector->currentItem() ] = text;
  mFindValidator->convert( mFindData, 
    mFindString[ mSelector->currentItem() ] );
}




void CFindDialog::defaultFocus( void )
{
  mInput->setFocus();
}


const char *CFindDialog::data( void )
{
  return( mFindData.size() == 0 ? 0 : mFindData.data() );
}


uint CFindDialog::size( void )
{
  return( mFindData.size() );
}


void CFindDialog::startFind( void )
{
  if( data() == 0 )
  {
    return;
  }

  USearchData sd;
  sd.mFind.operation   = USearchData::opFind;
  sd.mFind.pattern     = data();
  sd.mFind.size        = size();
  sd.mFind.step        = false;
  sd.mFind.backward    = mCheckBackward->isChecked();
  sd.mFind.fromCursor  = mCheckFromCursor->isChecked();
  sd.mFind.inSelection = mCheckInSelection->isChecked();
  emit execute( sd );
}


void CFindDialog::findAgain( void )
{
  if( data() == 0 )
  {
    return;
  }

  USearchData sd;
  sd.mFind.operation   = USearchData::opFind;
  sd.mFind.pattern     = data();
  sd.mFind.size        = size();
  sd.mFind.step        = true;
  sd.mFind.backward    = mCheckBackward->isChecked();
  sd.mFind.fromCursor  = true;
  sd.mFind.inSelection = mCheckInSelection->isChecked();
  emit execute( sd );
}















CReplaceDialog::CReplaceDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder ) 
{
  QVBoxLayout *topLayout = new QVBoxLayout( this, 5 ); 
  QHBoxLayout *inputRow;
  QLabel *label;

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  label = new QLabel( i18n("Format (find)"), this );
  label->setFixedHeight( label->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( label );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mFindSelector = new QComboBox( false, this );
  mFindSelector->setFixedHeight( mFindSelector->sizeHint().height() );
  mFindSelector->setMinimumWidth( mFindSelector->fontMetrics().width("M")*20 );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mFindSelector );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  label = new QLabel( i18n("Find"), this );
  label->setFixedHeight( label->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( label );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mFindInput = new QLineEdit( this );
  mFindInput->setFixedHeight( mFindInput->sizeHint().height() );
  mFindInput->setMinimumWidth( mFindInput->fontMetrics().width("M")*20 );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mFindInput );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  label = new QLabel( i18n("Format (replace)"), this );
  label->setFixedHeight( label->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( label );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mReplaceSelector = new QComboBox( false, this );
  mReplaceSelector->setFixedHeight( mReplaceSelector->sizeHint().height());
  mReplaceSelector->setMinimumWidth(
    mReplaceSelector->fontMetrics().width("M")*20);
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mReplaceSelector );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  label = new QLabel( i18n("Replace"), this );
  label->setFixedHeight( label->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( label );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mReplaceInput = new QLineEdit( this );
  mReplaceInput->setFixedHeight( mReplaceInput->sizeHint().height() );
  mReplaceInput->setMinimumWidth(
    mReplaceInput->fontMetrics().width("M") * 20 );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mReplaceInput );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  QButtonGroup *group = new QButtonGroup( i18n("Options"), this );  
  QGridLayout *gbox = new QGridLayout( group, 3, 2, 10 );
  gbox->addRowSpacing( 0, 0 );
  mCheckBackward = new QCheckBox( i18n("&Backwards"), group );
  mCheckBackward->setMinimumSize( mCheckBackward->sizeHint() );
  gbox->addWidget( mCheckBackward, 1, 0 );
  mCheckFromCursor = new QCheckBox( i18n("&From Cursor"), group );
  mCheckFromCursor->setMinimumSize( mCheckFromCursor->sizeHint() );
  gbox->addWidget( mCheckFromCursor, 2, 0 );
  mCheckInSelection = new QCheckBox( i18n("&In Selection"), group );
  mCheckInSelection->setMinimumSize( mCheckInSelection->sizeHint() );
  gbox->addWidget( mCheckInSelection, 1, 1 );
  mCheckPrompt = new QCheckBox( i18n("&Prompt"), group );
  mCheckPrompt->setMinimumSize( mCheckPrompt->sizeHint() );
  gbox->addWidget( mCheckPrompt, 2, 1 );
  gbox->activate();  
  inputRow->addSpacing( 5 );
  inputRow->addWidget( group );
  inputRow->addSpacing( 5 );

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 5, 5 );
  buttonBox->addStretch();
  QPushButton *replaceButton = buttonBox->addButton( i18n("&Apply"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("&Close"), false );
  buttonBox->layout();
  replaceButton->setDefault( true );
  buttonBox->setMinimumSize( replaceButton->sizeHint() );
  connect( replaceButton, SIGNAL(clicked()),this,SLOT(startReplace()) );
  connect( closeButton, SIGNAL(clicked()),this,SLOT(reject()) );
  topLayout->addWidget( buttonBox );


  topLayout->activate();
  topLayout->freeze( minimumSize().width(), minimumSize().height() );


  const char *items[] = 
  { 
    i18n( "Regular text" ),
    i18n( "Hexadecimal" ),
    i18n( "Octal" ),
    i18n( "Binary" ),
    0 
  };
  mFindSelector->insertStrList( items );
  mReplaceSelector->insertStrList( items );

  mFindValidator = new CHexValidator( this, CHexValidator::regularText );
  mReplaceValidator = new CHexValidator( this, CHexValidator::regularText );
  
  mFindInput->setValidator( mFindValidator );
  mReplaceInput->setValidator( mReplaceValidator );

  connect( mFindSelector, SIGNAL(activated(int)), 
	   SLOT(findSelectorChanged(int)) );
  connect( mFindInput, SIGNAL(textChanged(const char*)), 
	   SLOT(findInputChanged(const char*)) );
  connect( mReplaceSelector, SIGNAL(activated(int)), 
	   SLOT(replaceSelectorChanged(int)) );
  connect( mReplaceInput, SIGNAL(textChanged(const char*)), 
	   SLOT(replaceInputChanged(const char*)) );

  mCheckPrompt->setChecked( true );
  defaultFocus();
}


CReplaceDialog::~CReplaceDialog( void )
{
  delete mFindValidator;
  delete mReplaceValidator;
}




void CReplaceDialog::findSelectorChanged( int index )
{
  mFindValidator->setState( (CHexValidator::EState)index );
  mFindInput->setText( mFindString[ index ] );
}

void CReplaceDialog::findInputChanged( const char *text )
{
  mFindString[ mFindSelector->currentItem() ] = text;
  mFindValidator->convert( mFindData, 
    mFindString[ mFindSelector->currentItem() ] );
}

void CReplaceDialog::replaceSelectorChanged( int index )
{
  mReplaceValidator->setState( (CHexValidator::EState)index );
  mReplaceInput->setText( mReplaceString[ index ] );
}

void CReplaceDialog::replaceInputChanged( const char *text )
{
  mReplaceString[ mReplaceSelector->currentItem() ] = text;
  mReplaceValidator->convert( mReplaceData, 
    mReplaceString[ mReplaceSelector->currentItem() ] );
}

void CReplaceDialog::defaultFocus( void )
{
  mFindInput->setFocus();
}


const char *CReplaceDialog::findData( void )
{
  return( mFindData.size() == 0 ? 0 : mFindData.data() );
}

const char *CReplaceDialog::replaceData( void )
{
  return( mReplaceData.size() == 0 ? 0 : mReplaceData.data() );
}

uint CReplaceDialog::findSize( void )
{
  return( mFindData.size() );
}

uint CReplaceDialog::replaceSize( void )
{
  return( mReplaceData.size() );
}



void CReplaceDialog::startReplace( void )
{
  if( findData() == 0 )
  {
    return;
  }

  USearchData sd;
  sd.mReplace.operation   = USearchData::opReplace;
  sd.mReplace.pattern     = findData();
  sd.mReplace.pSize       = findSize();
  sd.mReplace.replacement = replaceData();
  sd.mReplace.rSize       = replaceSize();
  sd.mReplace.step        = false;
  sd.mReplace.backward    = mCheckBackward->isChecked();
  sd.mReplace.fromCursor  = mCheckFromCursor->isChecked();
  sd.mReplace.inSelection = mCheckInSelection->isChecked();
  sd.mReplace.prompt      = mCheckPrompt->isChecked();
  emit execute( sd );
  hide();
}





CReplacePromptDialog::CReplacePromptDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder )
{

  QBoxLayout *topLayout = new QVBoxLayout( this, 5 ); 
  QBoxLayout *inputRow;
  QLabel *label;

  topLayout->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  label = new QLabel( i18n("Replace data at cursor position?"), this );
  label->setFixedHeight( label->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( label );
  inputRow->addSpacing( 5 );

  topLayout->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  KSeparator *separator = new KSeparator( this );
  separator->setOrientation( QFrame::HLine );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( separator );
  inputRow->addSpacing( 5 );
  

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 5, 5 );
  buttonBox->addStretch();
  QPushButton *yesButton = buttonBox->addButton( i18n("Yes"), false );
  QPushButton *noButton = buttonBox->addButton( i18n("No"), false );
  QPushButton *allButton = buttonBox->addButton( i18n("All"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("Close"), false );
  connect( yesButton,SIGNAL(clicked()),this,SLOT(yes()));
  connect( noButton,SIGNAL(clicked()),this,SLOT(no()));
  connect( allButton,SIGNAL(clicked()),this,SLOT(all()));
  connect( closeButton, SIGNAL(clicked()),this,SLOT(close()) );
  buttonBox->layout();
  yesButton->setDefault( true );
  topLayout->addWidget( buttonBox );

  topLayout->activate();
  topLayout->freeze( minimumSize().width(), minimumSize().height() );
}


void CReplacePromptDialog::no( void ) 
{
  done( repNo );
}


void CReplacePromptDialog::yes( void ) 
{
  done( repYes );
}


void CReplacePromptDialog::all( void ) 
{
  done( repAll );
}


void CReplacePromptDialog::close( void ) 
{
  done( repClose );
}



void CReplacePromptDialog::done( int returnCode ) 
{
  setResult( returnCode );
  if( returnCode == repClose )
  {
    hide();
  }

  setResult( returnCode );
  emit execute( returnCode );
}









CInsertDialog::CInsertDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder ) 
{
  QVBoxLayout *topLayout = new QVBoxLayout( this, 5 ); 
  QHBoxLayout *inputRow;

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  QLabel *label = new QLabel( i18n("Size"), this );
  label->setFixedHeight( label->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( label );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mSizeBox = new QSpinBox( this );
  mSizeBox->setFixedHeight( mSizeBox->sizeHint().height() );
  mSizeBox->setMinimumWidth( mSizeBox->fontMetrics().width("M")*20 );
  mSizeBox->setRange( 1, 1000000 );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mSizeBox );
  inputRow->addSpacing( 5 );


  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  label = new QLabel( i18n("Format (pattern)"), this );
  label->setFixedHeight( label->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( label );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mPatternSelector = new QComboBox( false, this );
  mPatternSelector->setFixedHeight( mPatternSelector->sizeHint().height());
  mPatternSelector->setMinimumWidth(
    mPatternSelector->fontMetrics().width("M")*20);
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mPatternSelector );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  label = new QLabel( i18n("Pattern"), this );
  label->setFixedHeight( label->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( label );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mPatternInput = new QLineEdit( this );
  mPatternInput->setFixedHeight( mPatternInput->sizeHint().height() );
  mPatternInput->setMinimumWidth( mPatternInput->fontMetrics().width("M")*20 );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mPatternInput );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mOffsetLabel = new QLabel( i18n("Offset"), this );
  mOffsetLabel->setFixedHeight( mOffsetLabel->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mOffsetLabel );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mOffsetInput = new QLineEdit( this );
  mOffsetInput->setFixedHeight( mOffsetInput->sizeHint().height() );
  mOffsetInput->setMinimumWidth( mOffsetInput->fontMetrics().width("M")*20 );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mOffsetInput );
  inputRow->addSpacing( 5 );


  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  QButtonGroup *group = new QButtonGroup( i18n("Options"), this );  
  QGridLayout *gbox = new QGridLayout( group, 3, 2, 10 );
  gbox->addRowSpacing( 0, 0 );
  mCheckPattern = new QCheckBox( i18n("&Repeat pattern"), group );
  mCheckPattern->setMinimumSize( mCheckPattern->sizeHint() );
  gbox->addWidget( mCheckPattern, 1, 0 );
  mCheckOnCursor = new QCheckBox( i18n("&Insert On Cursor Position"), group );
  gbox->addWidget( mCheckOnCursor, 2, 0 );
  mCheckOnCursor->setMinimumSize( mCheckOnCursor->sizeHint() );
  connect( mCheckOnCursor, SIGNAL(clicked()),this,SLOT(cursorCheck()) );
  gbox->activate();
  inputRow->addSpacing( 5 );
  inputRow->addWidget( group );
  inputRow->addSpacing( 5 );
  

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 5, 5 );
  buttonBox->addStretch();
  QPushButton *applyButton = buttonBox->addButton( i18n("&Apply"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("&Close"), false );
  buttonBox->layout();
  applyButton->setDefault( true );
  buttonBox->setMinimumSize( applyButton->sizeHint() );
  connect( applyButton, SIGNAL(clicked()),this,SLOT(startInsert()) );
  connect( closeButton, SIGNAL(clicked()),this,SLOT(reject()) );
  topLayout->addWidget( buttonBox );

  topLayout->activate();
  topLayout->freeze( minimumSize().width(), minimumSize().height() );

  mCheckOnCursor->setChecked( true );
  cursorCheck();


  const char *items[] = 
  { 
    i18n( "Regular text" ),
    i18n( "Hexadecimal" ),
    i18n( "Octal" ),
    i18n( "Binary" ),
    0 
  };
  mPatternSelector->insertStrList( items );

  mPatternValidator = new CHexValidator( this, CHexValidator::regularText );
  mPatternInput->setValidator( mPatternValidator );

  connect( mPatternSelector, SIGNAL(activated(int)),
	   SLOT(patternSelectorChanged(int)) );
  connect( mPatternInput, SIGNAL(textChanged(const char*)), 
	   SLOT(patternInputChanged(const char*)) );

  defaultFocus();
}

CInsertDialog::~CInsertDialog( void )
{
  delete mPatternValidator;
}

void CInsertDialog::defaultFocus( void )
{
  mSizeBox->setFocus();
}




void CInsertDialog::patternSelectorChanged( int index )
{
  mPatternValidator->setState( (CHexValidator::EState)index );
  mPatternInput->setText( mPatternString[ index ] );
}

void CInsertDialog::patternInputChanged( const char *text )
{
  mPatternString[ mPatternSelector->currentItem() ] = text;
  mPatternValidator->convert( mPatternData, 
    mPatternString[ mPatternSelector->currentItem() ] );
}

const char *CInsertDialog::pattern( void )
{
  return( mPatternData.size() == 0 ? 0 : mPatternData.data() );
}

uint CInsertDialog::patternSize( void )
{
  return( mPatternData.size() );
}


void CInsertDialog::startInsert( void )
{
  SInsertData id;
  id.size = mSizeBox->value();
  id.repeatPattern = mCheckPattern->isChecked();
  id.onCursor = mCheckOnCursor->isChecked();
  id.pattern = pattern();
  id.patternSize = patternSize();

  if( id.onCursor == false )
  {
    bool success = stringToOffset( mOffsetInput->text(), id.offset );
    if( success == false )
    {
      return;
    }
  }

  execute( id );
  hide();
}


void CInsertDialog::cursorCheck( void )
{
  bool state = mCheckOnCursor->isChecked() == true ? false : true;
  mOffsetLabel->setEnabled( state );
  mOffsetInput->setEnabled( state );
}


void centerDialog( QWidget *widget, QWidget *centerParent )
{ 
  if( centerParent == 0 || widget == 0 )
  {
    return;
  }

  QPoint point = centerParent->mapToGlobal( QPoint(0,0) );
  QRect pos    = centerParent->geometry();

  widget->setGeometry( point.x() + pos.width()/2  - widget->width()/2,
		       point.y() + pos.height()/2 - widget->height()/2, 
		       widget->width(), widget->height() );
}



bool stringToOffset( const char *text, uint &offset )
{
  if( text == 0 || strlen(text) == 0 )
  {
    return( false );
  }

  const char *p = text;

  //
  // Skip any whitespaces in front of string
  //
  for( ; *p != 0 && isspace( *p ) ; p++ );
  
  int match = 0;
  bool space = false; 
  if( strncmp( p, "0x", 2 ) == 0 || strncmp( p, "0X", 2 ) == 0 )
  {
    for( const char *q = p+2; *q != 0; q++ )
    {
      if( isxdigit( *q ) == 0 || space == true )
      {
	if( isspace( *q ) == 0 )
	{
	  return( false );
	}
	space = true;
      }
    }
    match = sscanf( p+2, "%x", &offset );
  }
  else
  {
    for( const char *q = p; *q != 0; q++ )
    {
      if( isdigit( *q ) == 0 || space == true )
      {
	if( isspace( *q ) == 0 )
	{
	  return( false );
	}
	space = true;
      }
    }
    match = sscanf( p, "%u", &offset );
  }

  if( match == 0 )
  {
    return( false );
  }
  
  return( true );
}










CExportDialog::CExportDialog( QWidget *parent, const char *name )
  : QDialog( parent, name, false, WStyle_Customize|WStyle_DialogBorder ) 
{
  QVBoxLayout *topLayout = new QVBoxLayout( this, 5 ); 
  QHBoxLayout *inputRow;

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mDestinationLabel = new QLabel( i18n("Destination"), this );
  mDestinationLabel->setFixedHeight( mDestinationLabel->sizeHint().height() );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mDestinationLabel );
  inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  mDestinationInput = new QLineEdit( this );
  mDestinationInput->setFixedHeight( mDestinationInput->sizeHint().height() );
  mDestinationInput->setMinimumWidth( 
    mDestinationInput->fontMetrics().width("M")*20 );
  inputRow->addSpacing( 5 );
  inputRow->addWidget( mDestinationInput );
  inputRow->addSpacing( 5 );
  //QPushButton *browseButton = new QPushButton( i18n("&Browse") );
  //inputRow->addWidget( browseButton );
  //inputRow->addSpacing( 5 );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  QButtonGroup *group = new QButtonGroup( this );
  group->setFrameStyle( QFrame::NoFrame );  
  QGridLayout *gbox = new QGridLayout( group, 3, 1, 5 );
  gbox->addRowSpacing( 0, 0 );
  mRadioEverything = new QRadioButton( i18n("&Everything"), group );
  mRadioEverything ->setMinimumSize( mRadioEverything->sizeHint() );
  gbox->addWidget( mRadioEverything, 0, 0 );
  mRadioSelectedArea = new QRadioButton( i18n("&Selcted Area (lines)"), group);
  mRadioSelectedArea->setMinimumSize( mRadioSelectedArea->sizeHint() );
  gbox->addWidget( mRadioSelectedArea, 1, 0 );
  mRadioSpecify = new QRadioButton( i18n("Spec&ify (offset within line)"), 
				    group );
  mRadioSpecify->setMinimumSize( mRadioSpecify->sizeHint() );
  gbox->addWidget( mRadioSpecify, 2, 0 );
  gbox->activate();
  inputRow->addSpacing( 10 );
  inputRow->addWidget( group );
  inputRow->addSpacing( 10 );
  connect( group, SIGNAL(clicked(int)), SLOT(radioClicked(int)) );
  
  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  group = new QButtonGroup( this );
  group->setFrameStyle( QFrame::NoFrame );
  gbox = new QGridLayout( group, 2, 2, 5 );
  mFromLabel = new QLabel( i18n("From"), group );
  mFromLabel->setFixedHeight( mFromLabel->sizeHint().height() );
  gbox->addWidget( mFromLabel, 0, 0 );
  mToLabel = new QLabel( i18n("To"), group );
  mToLabel->setFixedHeight( mToLabel->sizeHint().height() );
  gbox->addWidget( mToLabel, 0, 1 );
  mFromInput = new QLineEdit( group );
  mFromInput->setFixedHeight( mFromInput->sizeHint().height() );
  gbox->addWidget( mFromInput, 1, 0 );
  mToInput = new QLineEdit( group );
  mToInput->setFixedHeight( mToInput->sizeHint().height() );
  gbox->addWidget( mToInput, 1, 1 );
  gbox->activate();
  inputRow->addWidget( group );

  inputRow = new QHBoxLayout();
  topLayout->addLayout( inputRow );
  group = new QButtonGroup( i18n("Options"), this );  
  gbox = new QGridLayout( group, 3, 1, 10 );
  gbox->addRowSpacing( 0, 0 );
  mCheckPS = new QCheckBox( i18n("Output As &PostScript"), group );
  mCheckPS->setMinimumSize( mCheckPS->sizeHint() );
  gbox->addWidget( mCheckPS, 1, 0 );
  mCheckStdout = new QCheckBox( i18n("&Dump To stdout"), group );
  mCheckStdout->setMinimumSize( mCheckStdout->sizeHint() );
  connect( mCheckStdout, SIGNAL(clicked()), SLOT(checkStdout()) );
  gbox->addWidget( mCheckStdout, 2, 0 );
  gbox->activate();
  inputRow->addSpacing( 5 );
  inputRow->addWidget( group );
  inputRow->addSpacing( 5 );

  KButtonBox *buttonBox = new KButtonBox( this, KButtonBox::HORIZONTAL, 5, 5 );
  buttonBox->addStretch();
  QPushButton *applyButton = buttonBox->addButton( i18n("&Apply"), false );
  QPushButton *closeButton = buttonBox->addButton( i18n("&Close"), false );
  buttonBox->layout();
  applyButton->setDefault( true );
  buttonBox->setMinimumSize( applyButton->sizeHint() );
  connect( applyButton, SIGNAL(clicked()), SLOT(startExport()) );
  connect( closeButton, SIGNAL(clicked()), SLOT(reject()) );
  topLayout->addWidget( buttonBox );

  topLayout->activate();
  topLayout->freeze( minimumSize().width(), minimumSize().height() );

  mRadioEverything->setChecked( true );
  radioClicked( 0 );
  mDestinationInput->setText( "hexdump.txt" );
  mCheckPS->setEnabled( false );

  defaultFocus();
}


CExportDialog::~CExportDialog( void )
{

}


void CExportDialog::defaultFocus( void )
{
  mDestinationInput->setFocus();
}


void CExportDialog::radioClicked( int id )
{
  bool state = id == 2 ? true : false; 
  mFromLabel->setEnabled( state );
  mToLabel->setEnabled( state );
  mFromInput->setEnabled( state );
  mToInput->setEnabled( state );
}

void CExportDialog::checkStdout( void )
{
  bool state = mCheckStdout->isChecked() ? false : true;
  mDestinationLabel->setEnabled( state );
  mDestinationInput->setEnabled( state );
}



void CExportDialog::startExport( void )
{
  SExportData ed;

  ed.url = mDestinationInput->text();

  if( mRadioEverything->isChecked() )
  {
    ed.mode = SExportData::Everything;
  }
  else if( mRadioSelectedArea->isChecked() )
  {
    ed.mode = SExportData::SelectedArea;
  }
  else if( mRadioSpecify->isChecked() )
  {
    ed.mode = SExportData::Specified;
    bool ok1 = stringToOffset( mFromInput->text(), ed.start );
    bool ok2 = stringToOffset( mToInput->text(), ed.stop );
    if( ok1 == false || ok2 == false )
    {
      QMessageBox::warning(
        this,
	i18n("Export"),
	i18n("Illegal format or missing offset specification"),
	i18n("OK"),
	"",
	"",
	0,0 );
      return;
    }
  }
  else
  {
    return;
  }

  ed.asPostScript = mCheckPS->isChecked();
  ed.toStdout = mCheckStdout->isChecked();

  ed.success = false;
  execute( ed );
  if( ed.success == true )
  {
    hide();
  }
}





